"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isUnDef = isUnDef;
exports.isDef = isDef;
exports.isNumber = isNumber;
exports.isString = isString;
exports.isArray = isArray;
exports.isObject = isObject;
exports.isFunction = isFunction;
exports.isEmptyObject = isEmptyObject;
exports.isPromise = isPromise;
exports.looseEqual = looseEqual;
exports.isEqual = exports.isFn = void 0;

/*!
 * 类型判断
 *
 * 整理参考
 * https://github.com/vuejs/vue/blob/dev/src/shared/util.js
 * https://github.com/enricomarino/is
 */

/**
 * 常用类型判断
 * 是否定义 字符串 数字 纯对象 空对象 数组 函数
 */
const objProto = Object.prototype; // const owns = objProto.hasOwnProperty;

const toString = objProto.toString; // 对象自身属性中是否具有指定的属性
// export function hasOwn(obj, prop) {
//   return owns.call(obj, prop);
// }

function isUnDef(v) {
  return v === 'undefined' || v === null;
}

function isDef(v) {
  return v !== 'undefined' && v !== null;
}

function isNumber(v) {
  return toString.call(v) === '[object Number]';
}

function isString(v) {
  return toString.call(v) === '[object String]';
}

function isArray(arr) {
  return Array.isArray(arr);
}

function isObject(v) {
  return v !== null && typeof v === 'object'; // && Array.isArray(v) === false;
}

function isFunction(v) {
  var isAlert = typeof window !== 'undefined' && v === window.alert;

  if (isAlert) {
    return true;
  }

  var str = toString.call(v);
  return str === '[object Function]' || str === '[object GeneratorFunction]' || str === '[object AsyncFunction]';
}

const isFn = isFunction;
exports.isFn = isFn;

function isEmptyObject(v) {
  return JSON.stringify(v) === '{}';
}

function isPromise(v) {
  return v && typeof v.then === 'function';
}
/**
 * looseEqual
 * Check if two values are loosely equal - that is,
 * if they are plain objects, do they have the same shape?
 *
 * @export
 * @param {*} a 比较值1
 * @param {*} b 比较值2
 * @returns {boolean} 布尔值
 */


function looseEqual(a, b) {
  if (a === b) return true;
  const isObjectA = isObject(a);
  const isObjectB = isObject(b);

  if (isObjectA && isObjectB) {
    try {
      const isArrayA = isArray(a);
      const isArrayB = isArray(b);

      if (isArrayA && isArrayB) {
        return a.length === b.length && a.every((e, i) => {
          return looseEqual(e, b[i]);
        });
      } else if (a instanceof Date && b instanceof Date) {
        return a.getTime() === b.getTime();
      } else if (!isArrayA && !isArrayB) {
        const keysA = Object.keys(a);
        const keysB = Object.keys(b);
        return keysA.length === keysB.length && keysA.every(key => {
          return looseEqual(a[key], b[key]);
        });
      } else {
        /* istanbul ignore next */
        return false;
      }
    } catch (e) {
      /* istanbul ignore next */
      return false;
    }
  } else if (!isObjectA && !isObjectB) {
    return String(a) === String(b);
  } else {
    return false;
  }
}

const isEqual = looseEqual;
exports.isEqual = isEqual;