"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sleep = sleep;
exports.random = random;
exports.randomRange = randomRange;
exports.uuid = uuid;
exports.forEachValue = forEachValue;
exports.cached = cached;
exports.merge = merge;
exports.throttle = throttle;
exports.debounce = debounce;
exports.camelCase = exports.kebabCase = exports.upperFirst = exports.hyphenate = void 0;

function sleep(timeout) {
  return new Promise(resolve => {
    setTimeout(() => {
      resolve(timeout);
    }, timeout);
  });
} // export function find(list, fn) {
//   return list.filter(fn)[0]
// }
// '_~0123456789' +
// 'abcdefghijklmnopqrstuvwxyz' +
// 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';


const randomString = '_~getRandomVcryp0123456789bfhijklqsuvwxzABCDEFGHIJKLMNOPQSTUWXYZ'; // https://github.com/ai/nanoid/blob/master/non-secure.js
// 指定范围，生成随机数

function random(size) {
  const result = [];

  while (0 < size--) {
    result.push(Math.floor(Math.random() * 256));
  }

  return result;
}

function randomRange(under, over) {
  switch (arguments.length) {
    case 1:
      return parseInt(Math.random() * under + 1);

    case 2:
      return parseInt(Math.random() * (over - under + 1) + under);

    default:
      return 0;
  }
}

function uuid(size = 21) {
  const url = randomString;
  let id = '';
  let bytes = [];

  if (typeof crypto !== 'undefined' && crypto.getRandomValues) {
    bytes = crypto.getRandomValues(new Uint8Array(size)); // console.warn(':::uuid crypto:', bytes.join(','));
  } else {
    bytes = random(size); // console.warn(':::uuid random:', bytes.join(','));
  }

  while (0 < size--) {
    id += url[bytes[size] & 63];
  }

  return id;
}

function forEachValue(obj, fn) {
  Object.keys(obj).forEach(key => fn(obj[key], key));
}
/**
 * Create a cached version of a pure function.
 *
 * @export
 * @param {*} fn 传入函数
 * @returns { function } 返回函数
 */


function cached(fn) {
  const cache = Object.create(null);
  return function cachedFn(str) {
    const hit = cache[str];
    /* eslint no-return-assign: 0 */

    return hit || (cache[str] = fn(str));
  };
} // 以下简单转化命名格式

/**
 * Camelize a hyphen-delimited string.
 * camelCase 小驼峰命名
 */


const camelizeRE = /-(\w)/g;
const camelize = cached(function (str) {
  /* eslint func-names: 0 */
  return str.replace(camelizeRE, function (_, c) {
    return c ? c.toUpperCase() : '';
  });
});
/**
 * Converts the first character of string to upper case.
 * 首字母大写
 */

const capitalize = cached(function (str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
});
/**
 * Hyphenate a camelCase string.
 * kebabCase 连字符命名 eg: kebab-case
 */

const hyphenateRE = /\B([A-Z])/g;
const hyphenate = cached(function (str) {
  return str.replace(hyphenateRE, '-$1').toLowerCase();
});
exports.hyphenate = hyphenate;
const upperFirst = capitalize;
exports.upperFirst = upperFirst;
const kebabCase = hyphenate;
exports.kebabCase = kebabCase;
const camelCase = camelize;
exports.camelCase = camelCase;

function merge(target) {
  for (let i = 1, j = arguments.length; i < j; i++) {
    const source = arguments[i] || {};

    for (const prop in source) {
      if (source.hasOwnProperty(prop)) {
        const value = source[prop];

        if (value !== undefined) {
          target[prop] = value;
        }
      }
    }
  }

  return target;
}

function throttle(func, wait, options) {
  let context;
  let args;
  let result;
  let timeout = null;
  let previous = 0;
  if (!options) options = {};

  const later = () => {
    previous = options.leading === false ? 0 : Date.now();
    timeout = null;
    result = func.apply(context, args);

    if (!timeout) {
      context = args = null;
    }
  };

  return (...rest) => {
    const now = Date.now();
    if (!previous && options.leading === false) previous = now;
    const remaining = wait - (now - previous);
    context = this;
    args = rest;

    if (remaining <= 0 || remaining > wait) {
      if (timeout) {
        clearTimeout(timeout);
        timeout = null;
      }

      previous = now;
      result = func.apply(context, args);

      if (!timeout) {
        context = args = null;
      }
    } else if (!timeout && options.trailing !== false) {
      timeout = setTimeout(later, remaining);
    }

    return result;
  };
}

function debounce(func, wait, immediate) {
  let timeout;
  let args;
  let context;
  let timestamp;
  let result;

  const later = () => {
    const last = Date.now() - timestamp;

    if (last < wait && last >= 0) {
      timeout = setTimeout(later, wait - last);
    } else {
      timeout = null;

      if (!immediate) {
        result = func.apply(context, args);
        if (!timeout) context = args = null;
      }
    }
  };

  return (...rest) => {
    context = this;
    args = rest;
    timestamp = Date.now();
    const callNow = immediate && !timeout;
    if (!timeout) timeout = setTimeout(later, wait);

    if (callNow) {
      result = func.apply(context, args);
      context = args = null;
    }

    return result;
  };
}